#!/usr/bin/env python
#-----------------------------------------------------------------------------
# Name:        mkplaylist.py
# Purpose:     Create playlists from directory scans.
#
# Author:      Marc 'BlackJack' Rintsch
#
# Created:     2004-11-09
# Copyright:   (c) 2004
# Licence:     GPL
#-----------------------------------------------------------------------------
"""Make a playlist file.

:var _EXTENSIONS: File extensions of media files.
"""

import sys
import os
import os.path

__author__ = "Marc 'BlackJack' Rintsch <marc(at)rintsch(dot)de>"
__version__ = '0.1.1'
__docformat__ = 'ReStructuredText'


_EXTENSIONS = ["." + _x for _x in ["mp3", "ogg", "wav", "flac",
                                   "ac3", "mod", "s3m", "it"]]

def isMediaFile(path):
    """Checks file if it is a known media file.
    
    The check is based on the filename extension and is case
    insensitive.
    
    :param path: filename of the file to check.
    :type path: str
    
    :returns: `True` if known media file, `False` otherwise.
    :rtype: bool
    """
    file_extension = os.path.splitext(path)[1].lower()
    for known_extension in _EXTENSIONS:
        if file_extension == known_extension:
            return True
    return False


def search(path):
    """Searches media files in `path` (recursivly).
    
    Symlinked directories are skipped to avoid endless
    scanning due to possible cycles in directory structure.
    
    :param path: root of the directory tree to search.
    :type path: str
    
    :returns: list of paths to the media files.
    :rtype: list
    """
    result = []
    for root, __, filenames in os.walk(path):
        print >> sys.stderr, "Scanning %s..." % root
        # TODO: Check if there are linked directories.
        for filename in filenames:
            if isMediaFile(filename):
                print >> sys.stderr, "Found", filename
                result.append(os.path.join(root, filename))
    
    return result


def main():
    """Main function."""
    from optparse import OptionParser
    
    outfile = sys.stdout
    
    usage = "usage: %prog [options] directory"
    parser = OptionParser(usage=usage, version=__version__)
    parser.add_option("-o", "--output", type="string", dest="outfile",
                      default='-',
                      help="Name of the output file or '-' for stdout")
    
    (options, args) = parser.parse_args()
    
    if len(args) != 1:
        parser.error("wrong number of arguments")

    if options.outfile != '-':
        outfile = file(options.outfile, "w")
    
    media_files = search(args[0])
    
    # TODO: Check if writing in a loop is faster.
    outfile.write('\n'.join(media_files))
    outfile.write('\n')
    outfile.close()
    
    print >> sys.stderr, "That's %d file(s)." % len(media_files)


if __name__ == '__main__':
    main()
